/*
	param.c -- Reads paramter file when program is started

  Poor Man's Packet (PMP)
  Copyright (c) 1991 by Andrew C. Payne    All Rights Reserved.

  Permission to use, copy, modify, and distribute this software and its
  documentation without fee for NON-COMMERCIAL AMATEUR RADIO USE ONLY is hereby
  granted, provided that the above copyright notice appear in all copies.
  The author makes no representations about the suitability of this software
  for any purpose.  It is provided "as is" without express or implied warranty.

	August, 1989
	Andrew C. Payne

	Modified January 12, 1992	Kevin Cozens
	Now checks for presence of an active level indicator when parsing
	the 'TXport' line in the config file. This will allow the PMPTEST
	program to work with either the regular PMP modem, the Baycom modem,
	or a PMP modem modified to be used via a serial port.
*/

/* ---- Includes ----- */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <stdarg.h>
#ifdef	PMPTEST
#define	EXTERN
#endif
#include "pmp.h"
#include "ports.h"

static int	dopttport(char *p), dotxport(char *p), docdport(char *p),
			dorxport(char *p), doinit(char *p), donormal(char *p),
			doinverse(char *p), dobright(char *p);

#ifndef	PMPTEST
static int	dostatus(char *p),	domessage(char *p),	domycall(char *p),
			dotxdelay(char *p),	dopthresh(char *p),	dofrack(char *p),
			docheck(char *p),	doretry(char *p),	domaxframe(char *p),
			dosound(char *p),	doctext(char *p),	dobtext(char *p),
			docapture(char *p),	dodefine(char *p),	dobeacon(char *p),
			dobaddr(char *p),	doautowrap(char *p),dopaclen(char *p),
			doclock(char *p);

void error(char *format, ...);
#endif

extern int SetAX25Addr(struct ax25_addr *p, char *c);

/* ----- Command list Structure ----- */

static struct param_cmd pcmds[] = {
	{ "pttport",	dopttport },
	{ "txport",	dotxport },
	{ "cdport",	docdport },
	{ "rxport",	dorxport },
	{ "init",	doinit },
	{ "normal",	donormal },
	{ "inverse",	doinverse },
	{ "bright",	dobright },
#ifndef	PMPTEST
	{ "status",	dostatus },
	{ "message",	domessage },
	{ "mycall",	domycall },
	{ "txdelay",	dotxdelay },
	{ "pthresh",	dopthresh },
	{ "frack",	dofrack },
	{ "retry",	doretry },
	{ "sound",	dosound },
	{ "ctext",	doctext },
	{ "btext", 	dobtext },
	{ "check",	docheck },
	{ "maxframe",	domaxframe },
	{ "capture", 	docapture },
	{ "define",	dodefine },
	{ "beacon",	dobeacon },
	{ "baddr",	dobaddr },
	{ "autowrap",	doautowrap },
	{ "paclen",	dopaclen },
	{ "clockskew",	doclock },
#endif
	{ NULL,		NULL }
};

static int	line;		/* current line number */


/* InitParameters()
	Sets initial parameters.
*/
void InitParameters()
{
	int	i;

#ifndef	PMPTEST
  #ifdef DEMO
	SetAX25Addr(&MyCall,"(DEMO)");
  #else
	SetAX25Addr(&MyCall,"NOCALL");
  #endif /* demo */
#endif
	ConnectTime = 0L;
	PrevConnectTime = 0L;
	TXStartFlags = 50;
	TXEndFlags = 2;
	Sound = FALSE;
	Pthresh = 0;
	Frack = 3;
	Retry = 7;
	Check = 60;
	MaxFrame = 7;
	Sound = TRUE;
	*btext = '\0';
	strcpy(baddr, "CQ");
	BeaconInt = 0;
	nctexts = 0;
	CaptureBuffer = 2048U;
	AutoWrap = 0;
	Paclen = 128;
	for (i = 0; i < MAXFKEYS; ++i)
		fkeys[i][0] = 0;

	Ppersist = 12000;
	Slottime = 2;
	Pwait = 0;
	ClockSkew = FALSE;

/* default I/O ports (based on LPT1:) */
	PTTPort = TXPort = 0x378;
	TXBit = 1;
	PTTBit = 2;

	CDPort = RXPort = 0x379;
	RXBit = 8;
	CDBit = 0x80;
	CDLevel = 0;
#ifdef	PMPTEST
	TXLevel = 0;
#endif
}

/* ReadParameters()
	Opens and reads the parameter file.

	Returns TRUE if error.
*/
int ReadParameters()
{
	FILE	*infile;
	char	s[100];
	char	cmd[80];
	char	*p;
	int	i;

/* open the file for input */
	if ((infile = fopen(ParamFname,"r")) == NULL)
	{
		sprintf(cmd, "Error -- Can't find configuration file '%s'", ParamFname);
		i = (80 - strlen(cmd)) / 2;
		gotoxy(i , 24);
		inverse();
		cputs(cmd);
		normal();
		return TRUE;
	}

/* read through lines till end of file */
	line = 0;
	while (fgets(s,100,infile) != NULL)
	{
		line++;
		p = sob(s);
		if (*p && *p != '#')
		{
			i = dispatch(p, pcmds);
#ifndef	PMPTEST
			if (i == FAIL)
			{
				p = extract(p, cmd);
				error("Unknown command '%s'",cmd);
			}
			if (i)
			{
				fclose(infile);
				return TRUE;
			}
#endif
		}
	}
	fclose(infile);
	return FALSE;
}

/* ----- Command Handlers ----- */

/* dopttport(s)
	Sets the port and bit for the PTT output signal.
*/
static int dopttport(char *s)
{
	PTTPort = (word)strtoul(s, &s, 0);
	s = sob(s);
	PTTBit = (byte)strtoul(s, &s, 0);
	s = sob(s);
	PTTLevel = (int)strtoul(s, &s, 0);
	return FALSE;
}

/* dotxport(s)
	Sets the port and bit for the TX data line. Also set TXLevel
	to same value as TXBit if the level value is a 1 to allow the
	transmitted data to be inverted as is required when using this
	software with the Baycom packet radio modem.
*/
static int dotxport(char *s)
{
	TXPort = (word)strtoul(s, &s, 0);
	s = sob(s);
	TXBit = (byte)strtoul(s, &s, 0);
#ifdef	PMPTEST
	s = sob(s);
	TXLevel = (byte)strtoul(s, &s, 0);
	if (TXLevel == 0)
		TXLevel = 0;
	else
		TXLevel = TXBit;
#endif
	return FALSE;
}

/* docdport(s)
	Sets the port and bit for the CD data line.
*/
static int docdport(char *s)
{
	CDPort = (word)strtoul(s, &s, 0);
	s = sob(s);
	CDBit = (byte)strtoul(s, &s, 0);
	s = sob(s);
	CDLevel = (int)strtoul(s, &s, 0);
	return FALSE;
}

/* dorxport(s)
	Sets the port and bit for the RX data line.
*/
static int dorxport(char *s)
{
	RXPort = (word)strtoul(s, &s, 0);
	s = sob(s);
	RXBit = (byte)strtoul(s, &s, 0);
	return FALSE;
}

/* doinit(s)
	Send an initializing byte value to a port upon startup.
*/
static int doinit(char *s)
{
	word port, value;

	port = (word)strtoul(s, &s, 0);
	s = sob(s);
	value = (word)strtoul(s, &s, 0);
	outportb(port, value);
	return FALSE;
}

/* donormal(s)
	Sets the normal screen attribute.
*/
static int donormal(char *s)
{
	NormalAttr = atoi(s);
	return FALSE;
}

/* doinverse(s)
	Sets the inverse screen attribute.
*/
static int doinverse(char *s)
{
	InvAttr = atoi(s);
	return FALSE;
}

/* dobright(s)
	Sets the bright screen attribute
*/
static int dobright(char *s)
{
	BrightAttr = atoi(s);
	return FALSE;
}


#ifndef	PMPTEST

/* dostatus(s)
	Sets the status line attribute
*/
static int dostatus(char *s)
{
	StatusAttr = atoi(s);
	return FALSE;
}

/* domessage(s)
	Sets the message line attribute
*/
static int domessage(char *s)
{
	MsgAttr = atoi(s);
	return FALSE;
}

/* domycall(s)
	Sets the callsign of this node.
*/
static int domycall(char *s)
{
#ifndef	DEMO
	if (SetAX25Addr(&MyCall,s))
	{
		error("Invalid callsign");
		return TRUE;
	}
#endif /* demo */

	return FALSE;
}

/* dotxdelay(s)
	Sets the transmitter delay.  This is the wait after the TX is keyed
	and before data is sent.  The time is given in milliseconds.
*/
static int dotxdelay(char *s)
{
	TXStartFlags = max(2, (int)((long)atoi(s) * 1200L / 8000L) );
	return FALSE;
}

/* dopthresh(s)
	This parameter is a slight modification to AX.25 Version 2.0.  If
	an I-packet has a data field smaller than this parameter, it is
	always resent w/ a poll after a timeout, instead of polling w/ RR.
*/
static int dopthresh(char *s)
{
	Pthresh = atoi(s);
	return FALSE;
}

/* dofrack(s)
	This parameter is used to calculate the timeout time before retrying.
*/
static int dofrack(char *s)
{
	Frack = atoi(s);
	return FALSE;
}

/* docheck(s)
	This parameter is used as a keep-alive timer (in seconds).
*/
static int docheck(char *s)
{
	Check = atoi(s);
	return FALSE;
}

/* doretry(s)
	This parameter specifies the maximum number of retries before a
	connection is dropped.
*/
static int doretry(char *s)
{
	Retry = atoi(s);
	return FALSE;
}

/* domaxframe(s)
	This parameter specifies the maximum number of frames allowed
	outstanding (e.g. unACKed).
*/
static int domaxframe(char *s)
{
	MaxFrame = atoi(s);
	if (MaxFrame < 1 || MaxFrame > 7)
	{
		error("MAXFRAME must be between 1 and 7");
		return TRUE;
	}

	return FALSE;
}

/* dosound(s)
	Enables/disables sound.
*/
static int dosound(char *s)
{
	Sound = atoi(s);
	return FALSE;
}

/* doctext(s)
	Sets CTEXT lines, the connect welcome string.
*/
static int doctext(char *s)
{
	if (nctexts >= MAXCTEXTS)	/* check for too many lines */
	{
		error("Too many CTEXT lines.");
		return TRUE;
	}

	ctext[nctexts++] = strdup(s);
	return FALSE;
}

/* dobtext(s)
	Sets btext, the beacon/CQ string.

	btext includes a carriage return.
*/
static int dobtext(char *s)
{
	btext = strdup(s);
	return FALSE;
}

/* docapture(s)
	This buffer specifies the size of the capture buffer.  The larger this
	number, the less disk writes required.  This number should be a multiple
	of a sector size (512).
*/
static int docapture(char *s)
{
long temp;

	temp = atol(s);
	if (temp < 512L || temp > 65536L)
	{
		error("CAPTURE must be between 512 and 65536 bytes");
		return TRUE;
	}
	CaptureBuffer = (word)temp;
	return FALSE;
}

/* dodefine(s)
	Defines function keys
*/
static int dodefine(char *s)
{
#ifndef	PMPTEST
	char	keyname[80];
	int	keynumber;

/* extract function key */
	s = extract(s, keyname);
	keynumber = atoi(keyname + 1);
	if (*keyname != 'f' || keynumber < 0 || keynumber > MAXFKEYS)
	{
#if	defined(TRACE) || defined(DEBUG)
		error("Function key F1-F8 expected");
#else
		error("Function key F1-F10 expected");
#endif
		return TRUE;
	}

/* extract key definition */
	if (convertkey(fkeys[keynumber-1], s))
	{
		error("Error in definition for function key %s",keyname);
		return TRUE;
	}
#endif
	return FALSE;
}

/* dobeacon(s)
	Sets the interval for beaconing (in seconds).
*/
static int dobeacon(char *s)
{
	BeaconInt = atoi(s);
	return FALSE;
}

/* dobaddr(s)
	Sets the destination call and path for beacon packets.
*/
static int dobaddr(char *s)
{
	baddr = strdup(s);
	return FALSE;
}

/* doautowrap(s)
	Sets the column at which automatic word wrap is active.
*/
static int doautowrap(char *s)
{
	AutoWrap = atoi(s);
	if (AutoWrap && (AutoWrap < 10 || AutoWrap > 79))
	{
		error("AUTOWRAP must be zero or between 10 and 79 bytes");
		return TRUE;
	}
	return FALSE;
}

/* dopaclen(s)
	Sets the maximum transmitted packet length.
*/
static int dopaclen(char *s)
{
	Paclen = atoi(s);
	if (Paclen < 1 || Paclen > 256)
	{
		error("PACLEN must be between 1 and 256 bytes");
		return TRUE;
	}
	return FALSE;
}

/* doclock(s)
	Enables or disables attempted clock skew adjust.
*/
static int doclock(char *s)
{
	ClockSkew = atoi(s);
	return FALSE;
}


/* error()
	Show error message.
*/
void error(char *format, ...)
{
	char	s[100];
	va_list	argptr;

	cprintf("Error on line %d in configuration file PMP.CFG :\r\n",line);
	va_start(argptr, format);
	vsprintf(s, format, argptr);
	va_end(argptr);
	cprintf(s);
}

#endif	/* #ifndef PMPTEST */
