/*
	pmp.c -- main program

  Poor Man's Packet (PMP)
  Copyright (c) 1991 by Andrew C. Payne    All Rights Reserved.

  Permission to use, copy, modify, and distribute this software and its
  documentation without fee for NON-COMMERCIAL AMATEUR RADIO USE ONLY is hereby
  granted, provided that the above copyright notice appear in all copies.
  The author makes no representations about the suitability of this software
  for any purpose.  It is provided "as is" without express or implied warranty.

	July, 1989
	Andrew C. Payne
*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <alloc.h>
#include <mem.h>
#include <bios.h>
#include <dos.h>
#include <io.h>
#include <ctype.h>
#include <time.h>
#include <string.h>

#define EXTERN

#include "keys.h"
#include "pmp.h"
#include "ports.h"

static void usage(void);
static void TitleScreen(void);
static void MainLoop(void);
static void wait(void);
static void DoHelp(void);
#ifdef	TRACE
static void DoDebug(void);
#endif
#ifndef	DEMO
static void DoConnect(void);
#endif
static void DoStatus(void);
static void DoPause(void);
#ifndef	DEMO
static void DoUpload(void);
#endif
static void DoCapture(void);
static void DoSnapshot(void);
FILE *OpenFile(char *, char *, int);
void time2str(long, char *);


/* ----- Title Screen ----- */

/* TitleScreen()
	Shows the title screen.
*/
static void TitleScreen(void)
{
	normal();
	clrscr();
	cprintf("                                                             \r\n");
	cprintf("                                                             \r\n");
	cprintf("                                         \r\n");
	cprintf("                                             \r\n");
	cprintf("                                             \r\n");
	cprintf("                                      \r\n");
	cprintf("                                                     \r\n");
	cprintf("                                                     \r\n");
	cprintf("            oor                  an's     acket      \r\n");
	cprintf("                                                             \r\n\n");

	bright();
	cprintf("                A Complete Packet Radio Program For The IBM PC\r\n\n");
	normal();

#ifdef DEMO
	cprintf("                       Version %s (DEMO) %s \r\n\n\n", VERSION, __DATE__);
#else
	cprintf("                         Version %s  %s \r\n\n\n", VERSION, __DATE__);
#endif
	cprintf("                           By Andrew C. Payne, N8KEI\r\n");
	cprintf("                  with special thanks to Kevin Feeney, WB2EMS\r\n\n");
	cprintf("           Additional modifications courtesy of Kevin Cozens, VE3SYB\r\n\n");
	cprintf("                       Copyright (c) 1989, 1990, 1991 ACP\r\n");
	cprintf("                              All rights reserved.\r\n");
}

/* ----- Subroutines ----- */

/* stoupper(s)
	Convert string to upper case.
*/
void stoupper(char *s)
{
	while (*s)
	{
		*s = toupper(*s);
		s++;
	}
}

/* wait()
	Prompts user for keystroke.
*/
static void wait(void)
{
	bright();
	gotoxy(1, 25);
	cprintf("                     ----- Press any key to continue ------");
	(void)getkey();
}

/* WaitKey(p)
	Waits for a keystroke, calling PeriodicHook() while waiting.
	Returns keystroke.  If 'p' is non-zero, then a prompt is shown
	on line 'p'.
*/
KEY WaitKey(int prompt)
{
	if (prompt)
		CenterTitle(prompt, "  Press any key to resume  ");

	while (!keypressed())
		PeriodicHook();

	return getkey();
}

/* ----- Sound Control ----- */

/* StartSound(freq, time)
	Starts a sound of the given frequency and duration (in 50 ms BIOS
	clock ticks).
*/
void StartSound(int freq, int time)
{
	if (Sound)
	{
		sound(freq);
		SoundEnd = BiosTime() + time;
	}
}

/* ----- Upper Level User Commands ------ */

/* DoHelp()
	Handles the help screen.
*/
void DoHelp(void)
{
	SaveScreen(TRUE, TRUE);
	CenterTitle(1, "   PMP Help   ");
	GotoXY(1, 2);
	_uputs(NormalAttr, "\n");
#if	defined(TRACE) || defined(DEBUG)
	_uputs(NormalAttr, "   ALT-B  Send Beacon                        F1-F8 User defined macros\n");
#else
	_uputs(NormalAttr, "   ALT-B  Send Beacon                       F1-F10 User defined macros\n");
#endif
	_uputs(NormalAttr, "   ALT-C  Connect\n");
	_uputs(NormalAttr, "   ALT-D  Disconnect                      HOME/END Start or end of scrollback\n");
	_uputs(NormalAttr, "   ALT-H  Show help (this screen)          UP/DOWN Scrollback line at a time\n");
	_uputs(NormalAttr, "   ALT-J  Screen shapshot to file          PGUP/DN Scrollback page at a time\n");
	_uputs(NormalAttr, "   ALT-L  Download/capture text to file\n");
	_uputs(NormalAttr, "   ALT-N  Nodes recently heard\n");
	_uputs(NormalAttr, "   ALT-P  Pause screen\n");
	_uputs(NormalAttr, "   ALT-S  Show system status\n");
	_uputs(NormalAttr, "   ALT-U  Upload a text file\n");
	_uputs(NormalAttr, "   ALT-W  Write scrollback to disk\n");
	_uputs(NormalAttr, "   ALT-X  Exit PMP\n\n");
	_uputs(NormalAttr, "   \n\n");
	_uputs(NormalAttr, "        Comments, questions, and money to:\n");
	_uputs(NormalAttr, "              Andrew C. Payne\n");
	_uputs(NormalAttr, "              Route 3, Box 78-Q\n");
	_uputs(NormalAttr, "              Berkeley Springs, WV  25411");
	(void)WaitKey(23);
	RestoreScreen();
}

/* DoDebug()
	Toggle debug mode on and off.
*/
void DoDebug(void)
{
	DebugMode = !DebugMode;
	putstring(80, 25, 1, InvAttr, DebugMode ? "" : " ");
}

/* DoConnect()
	Prompts user for connect path, starts connection.
*/
#ifndef DEMO
static void DoConnect(void)
{
	char	path[80];

/* if we are disconnected, prompt for connect path and initiate connect */
	if (AX25_Control.state == DISCONNECTED)
	{
		if (GetInput("Connect to --> ", path, 60))
		{
			memcpy(&AX25_Control.header.source, 
				&MyCall, sizeof(struct ax25_addr));
			AX25_Control.header.pid = PID_TEXT;
			AX25_Control.type = TEXT;
			if (SetAX25Path(path, &AX25_Control.header))
				AX25_Open();
		}
	}
}
#endif /* demo */

/* DoStatus()
	Shows the status of the node, primarily the health counters.
*/
static void DoStatus(void)
{
	char	s[80];
	long	t;
	struct tm	*tm;
#ifdef TRACE
	extern int	nlogs;
#endif

	SaveScreen(TRUE, TRUE);
	CenterTitle(1, "   PMP Status   ");

	time(&t);

	tm = localtime(&t);
	sprintf(s, "     Status at %02d/%02d/%02d %02d:%02d:%02d",
			tm->tm_mon+1, tm->tm_mday, tm->tm_year,
			tm->tm_hour,  tm->tm_min,  tm->tm_sec);
	GotoXY(1, 4);
	_uputs(NormalAttr, s);

	t -= StartTime;
	strcpy(s, "Running time ");
	time2str(t, &s[ strlen(s) ]);
	GotoXY(48, 4);
	_uputs(NormalAttr, s);

	if (ConnectTime == 0L)
		t = ConnectTime;
	else
		t = time(NULL) - ConnectTime;
	strcpy(s, "Connected for ");
	time2str(t, &s[ strlen(s) ]);
	GotoXY(47, 6);
	_uputs(NormalAttr, s);

	t = PrevConnectTime;
	strcpy(s, "Last connected for ");
	time2str(t, &s[ strlen(s) ]);
	GotoXY(42, 7);
	_uputs(NormalAttr, s);

	GotoXY(1, 6);
	sprintf(s, "      %8ld good frames received\n", RXCount);
	_uputs(NormalAttr, s);
	sprintf(s, "      %8ld framing errors\n", RXFrameErr);
	_uputs(NormalAttr, s);
	sprintf(s, "      %8ld checksum errors\n\n", RXCRCErr);
	_uputs(NormalAttr, s);
	sprintf(s, "      %8ld receive buffer overflows\n", RXBOverflow);
	_uputs(NormalAttr, s);
	sprintf(s, "      %8ld receive queue overflows\n", RXQOverflow);
	_uputs(NormalAttr, s);
	sprintf(s, "      %8ld REJ frames received\n", RXREJ);
	_uputs(NormalAttr, s);
	sprintf(s, "      %8ld FRMR frames received\n\n", RXFRMR);
	_uputs(NormalAttr, s);
	sprintf(s, "      %8ld frames transmitted\n\n", TXCount);
	_uputs(NormalAttr, s);
#ifdef TRACE
	sprintf(s, "      %8ld frames in trace log\n\n", (long)nlogs);
	_uputs(NormalAttr, s);
#endif
	sprintf(s, "  %12ld bytes free\n\n", coreleft());
	_uputs(NormalAttr, s);

	if (Capturing())
	{
		sprintf(s, "     Capturing to %s (%u bytes captured)",
				CaptureFile, CaptureSize);
		_uputs(NormalAttr, s);
	}

	(void)WaitKey(23);
	RestoreScreen();
}

/* DoPause()
	Pause the screen.
*/
void DoPause(void)
{
	SaveScreen(TRUE, FALSE);
	(void)WaitKey(23);
	RestoreScreen();
}

/* DoUpload()
	Uploads a file.

	Crude first version, error handling sucks.
*/
#ifndef DEMO
static void DoUpload(void)
{
	char	fname[40];		/* file name to upload */
	byte	inbuf[256];		/* input buffer */
	char	text[90];		/* temp text area */
	long	filesize;		/* length of file in bytes */
	long	starttime;		/* start time of upload */
	long	uploadtime;		/* time used to upload */
	FILE	*upfile;
	int	cursor;
	int	len;			/* # of bytes acutally read */

/* can't upload unless connected */
	if (!Connected())
	{
		sound(400);
		delay(200);
		nosound();
		return;
	}

/* get filename, open file */
	if (GetInput("Upload what file? --> ", fname, 40))
	{
		if (!*fname)
			return;

		stoupper(fname);
		upfile = fopen(fname, "r");
		if (upfile == NULL)		/* error on open */
		{
			uprintf(InvAttr, "  Error opening '%s' : %s  \n", fname, sys_errlist[errno]);
			return;
		}

/* show upload status */
		filesize = filelength(fileno(upfile));
		cursor = cursave();
		curoff();
		sprintf(text, "    Uploading %-40s   Press <ESC> to abort", fname);
		putstring(1, 24, 80, InvAttr, text);
		starttime = BiosTime();

/* read lines from file */
		while (len = fread(inbuf, 1, 256, upfile))
		{

/* wait for space in AX.25 queue, then enqueue the data */
			do {
				PeriodicHook();		/* wait to empty */
				if (keypressed() && (getkey() == ESC))
				{
					Notify("    Upload aborted.");
					goto done;
				}
			} while (AX25QFull());

			LinkSend((byte *)inbuf, len);		/* send the data */
			uputtext(BrightAttr, inbuf, len);
		}
		uploadtime = (BiosTime() - starttime) / (long)BIOSSEC;
		if (uploadtime == 0L)
			uploadtime = 1L;

/* if terminated for any reason other than EOF, handle error */
		if (ferror(upfile))
		{
			uprintf(InvAttr, " Error reading '%s' : %s  \n", 
				fname, sys_errlist[errno]);
		}
		else
		{
			if (Sound)			/* two beeps */
			{
				sound(2600);
				delay(100);
				nosound();
				delay(50);
				sound(2600);
				delay(100);
				nosound();
			}
			sprintf(text, "    Upload complete.  (%ld seconds for %ld bytes = %ld bytes/sec)", 
					uploadtime, filesize, filesize/uploadtime);
			Notify(text);
		}
done:
		clear_area(24, 1, 80);		/* clean up */
		currest(cursor);
		fclose(upfile);
	}
}
#endif /* demo */

/* DoCapture()
	Toggles file capture (log to disk).

	Note:   The test for a file aready existing could be cleaned up
		using 'access'.
*/
void DoCapture(void)
{
	char	text[80];               /* temp text area */

/* If already capturing, close file */
	if (Capturing())
	{
		sprintf(text, "    Closing capture file  %s   (%u bytes)",
				CaptureFile, CaptureSize);
		Notify(text);
		CloseCapture();
		putstring(60, 25, 3, StatusAttr, "");
		return;
	}

	OpenCapture();
}

/* DoSnapshot()
	Write a snapshot of the current screen to a file.
*/
void DoSnapshot(void)
{
	char	fname[41];		/* capture snapshot */
	FILE	*snapshot;
	char	temp[160];		/* temp area */
	char	line[81];		/* output line */
	int		i, l;			/* screen line */
	char	*p, *q;

/* open snapshot file*/
	snapshot = OpenFile("Enter name for snapshot file> ", fname, sizeof(fname));
	if (snapshot == NULL)
		return;

/* write contents of screen to file */
	for (l = 1; l < 24; ++l)
	{
		gettext(1, l, 80, l, temp);
		q = line;
		p = temp;
		for (i = 0; i < 80; ++i)	/* extract contents */
		{
			*q++ = *p++;
			p++;
		}
		do {			/* trim line */
			q--;
		} while (q >= line && (*q == '\0' || *q == ' '));
		*++q = '\n';
		*++q = '\0';
		fputs(line, snapshot);	/* should error check this */
	}
	fclose(snapshot);
}

/* ----- Main Loop ----- */

/* PeriodicHook()
	When in an idle loop, this routine should be called as much as
	possible.  If the carrier is down, goes to RX, handles timers
	and flushes the TX queue.
*/
void PeriodicHook(void)
{
	long	t;

/* Go to RX if a carrier is detected */
	while (RXCarrier())
	{
		if (RXLevel1()) 		/* receive incoming */
			RXProcess();	/* process incoming */
		Pwait = 0;
	}

#ifndef DEMO
/* process AX.25 link transmit items, if any */
	AX25_Flush();

/* check for expired timers */
	t = BiosTime();
	if (AX25_Control.t1 != 0 && AX25_Control.t1 <= t)
			AX25_Expire(1);
	if (AX25_Control.t3 != 0 && AX25_Control.t3 <= t)
			AX25_Expire(3);
	if (SoundEnd && t >= SoundEnd)		/* sound timeout */
	{
		nosound();
		SoundEnd = 0;
	}
	if (BeaconEnd && t >= BeaconEnd)	/* beacon timeout */
	{
		SendBeacon();
		StartBeacon();
	}

/* do p-persistence, and flush anything in the TX queue */
	if (Pwait == 0 || t > Pwait)	/* attempt transmission */
	{
		if (rand() < Ppersist)	/* we are a go? */
		{
			Pwait = 0;
			TXQEmpty();		/* Transmit! */
		} else
			Pwait = t + Slottime;	/* wait a bit */
	}
#endif /* demo */
}

/* MainLoop()
	Main command handling loop.
*/
static void MainLoop(void)
{
	char	buf[82];	/* input buffer */
	char	temp;		/* temp character storage */
	int	p;				/* position in input buffer */
	KEY	k;				/* keypress */
	byte	c;
	int	state;			/* current connected state */
	int	scrollback;		/* TRUE if scrolling back */
	int	i;

/* initalize */
	InitKeybuffer();
	scrollback = FALSE;
	p = 0;
	gotoxy(1, 24);
	state = Connected();

/* main loop */
	while (TRUE)
	{

/* loop until keypress or change in state */
		while (!keypressed() && (state == Connected()))
			PeriodicHook();		/* do RX, TX, timers */
		state = Connected();

/* handle the cursor */
		if (Connected())		/* if connected, show cursor */
			curon();
		else
		{
			if (p)				/* clear any text */
			{
				clear_area(24, 1, 80);
				p = 0;
			}
			curoff();
		}

/* handle any user keystrokes */
		while (k = Nextkey())
		{
			/* exit scrollback mode, if appropriate */
			if (scrollback)
			{
				if (k != ALTJ && k != UP && k != DOWN &&
					k != PGDN && k != PGUP && k != HOME)
				{
					EndScrollback();
					scrollback = FALSE;
				}
			}

/* ASCII keypress, show on bottom line */
			if ((c = asciicode(k)) && Connected())
			{
#ifndef DEMO
				bright();
				switch (c)
				{
				case '\r':		/* send */
					buf[p++] = '\n';
					buf[p] = '\0';
					uputs(BrightAttr, buf);
					LinkSend((byte *)buf, p);
					clear_area(24, 1, 80);
					p = 0;
					break;
				case asciicode(CAN):
					clear_area(24, 1, 80);
					p = 0;
					break;
				case '\b':		/* backspace */
					if (p > 0)
					{
						cprintf("\b \b");
						p--;
					}
					break;
				case ' ':		/* test for autowrap */
					if (AutoWrap && p > AutoWrap)
					{
						i = p - 1;
						while (i && buf[i] != ' ')
							i--;
						if (i)	/* wrap word */
						{
							buf[i++] = '\n';
							temp = buf[i];
							buf[i] = '\0';
							LinkSend((byte *)buf, i);
							uputs(BrightAttr, buf);
							buf[i] = temp;
							buf[p] = '\0';
							strcpy(buf, &buf[i]);
							p -= i;
							clear_area(24, 1, 80);
							if (p)
								putstring(1, 24, 80, BrightAttr, buf);
						}
					}
					/* note fall through */
				default:		/* char */
					if (p < 79)
						putch(buf[p++] = c);
				}
#endif /* demo */

/* else, function key */
			}
			else
			{
				switch (k)
				{
				case DOWN:		/* scroll */
					if (scrollback)
					{
						if (MoveScrollback(1))
						{
							EndScrollback();
							scrollback = FALSE;
						}
					}
					break;
				case UP:		/* scroll */
					if (!scrollback)
						scrollback = StartScrollback();
					if (scrollback)
						MoveScrollback(-1);
					break;
				case PGDN:		/* scroll */
					if (scrollback)
					{
						if (MoveScrollback(23))
						{
							EndScrollback();
							scrollback = FALSE;
						}
					}
					break;
				case PGUP:		/* scroll */
					if (!scrollback)
						scrollback = StartScrollback();
					if (scrollback)
						MoveScrollback(-23);
					break;
				case HOME:		/* scroll */
					if (!scrollback)
						scrollback = StartScrollback();
					if (scrollback)
						MoveScrollback(-32000);
					break;
				case ALTX:		/* exit */
					if (!Connected())
						return;
					else
					{
						sound(400);
						delay(200);
						nosound();
					}
					break;
#ifndef DEMO
				case ALTB:		/* beacon */
					SendBeacon();
					break;
				case ALTC:		/* connect */
					DoConnect();
					break;
				case ALTD:		/* discnnct */
					AX25_Close();
					break;
#endif /* demo */
				case ALTH:		/* help */
					DoHelp();
					break;
				case ALTJ:		/* snapshot */
					SaveMessage();
					DoSnapshot();
					RestoreMessage();
					state = -1;
					break;
				case ALTN:		/* nodes heard */
					DoHeard();
					break;
				case ALTP:		/* pause */
					DoPause();
					break;
				case ALTS:		/* node status */
					DoStatus();
					break;
#ifndef DEMO
				case ALTU:		/* file upload */
					SaveMessage();
					DoUpload();
					RestoreMessage();
					state = -1;
					break;
#endif
				case ALTL:		/* file download/capture */
					SaveMessage();
					DoCapture();
					RestoreMessage();
					state = -1;
					break;
				case ALTW:		/* write scrollback */
					SaveMessage();
					WriteScrollback();
					RestoreMessage();
					state = -1;
					break;
				case F1:
					AddKeystrokes(fkeys[0]);
					break;
				case F2:
					AddKeystrokes(fkeys[1]);
					break;
				case F3:
					AddKeystrokes(fkeys[2]);
					break;
				case F4:
					AddKeystrokes(fkeys[3]);
					break;
				case F5:
					AddKeystrokes(fkeys[4]);
					break;
				case F6:
					AddKeystrokes(fkeys[5]);
					break;
				case F7:
					AddKeystrokes(fkeys[6]);
					break;
				case F8:
					AddKeystrokes(fkeys[7]);
					break;
#if	!defined(DEBUG) && !defined(TRACE)
				case F9:
					AddKeystrokes(fkeys[8]);
					break;
				case F10:
					AddKeystrokes(fkeys[9]);
					break;
#else
  #ifdef DEBUG
				case F9:		/* toggle PASSALL */
					PassMode = !PassMode;
					break;
  #endif
  #ifdef TRACE
				case F10:		/* debug */
					DoDebug();
					break;
  #endif
#endif	/* !defined(DEBUG) && !defined(TRACE) */
				}
			}
			gotoxy(p+1, 24);
		}
	}
}

/* ----- Error handling ----- */

/* OutOfMemory()
	Gets called during initialization routines to show an out of
	memory error.  Prompts for user keystroke and exits.
*/
void OutOfMemory(void)
{
	cprintf("  Fatal Error -- Out of Memory!  ");
	wait();
	CRTExit();
	exit(-1);
}

/* usage()
	Show the command line usage.
*/
static void usage(void)
{
	printf("Usage:  PMP [-a] [-p<paramfile>]\n");
}

/* ----- Main Program ----- */

void main(int argc, char **argv)
{
	int	i;

	/* Initialize status counters */
	RXCount = 0L;
	RXQOverflow = 0L;
	RXBOverflow = 0L;
	RXFrameErr = 0L;
	RXCRCErr = 0L;
	RXFRMR = 0L;
	RXREJ = 0L;
	TXCount = 0L;
	memset(&AX25_Control, 0, sizeof(AX25_Control));

/* process command line parameters */
	AutoMode = FALSE;
	strcpy(ParamFname, "PMP.CFG");
	for (i = 1; i < argc; ++i)
	{
		if (argv[i][0] != '-')
		{
			usage();
			exit(-1);
		}

		switch (toupper(argv[i][1]))
		{
		case 'A':
			AutoMode = TRUE;
			break;
		case 'P':
			if (argv[i][2] != '\0')
				strcpy(ParamFname, argv[i]+2);
			else
			{
				if (i < argc - 1)
					strcpy(ParamFname, argv[++i]);
				else
				{
					usage();
					exit(0);
				}
			}
			break;
		default:
			printf("Unknown switch: %s\n", argv[i]);
			usage();
			exit(0);
		}
	}

	/* check to see if we've got enough memory to breathe */
	if (coreleft() < 65536L)
	{
		printf("Not enough free memory to run PMP.\n");
		exit(-1);
	}

	InitParameters();		/* Initialize default system parameters */
#ifndef DEMO
	TXKey(FALSE);			/* no transmitter */
#endif
	CRTInit();
	TitleScreen();

/* read the parameter file */
	gotoxy(1, 23);
	inverse();				/* set up for errors */
	if (ReadParameters())	/* if error reading parameter file */
	{
		wait();
		CRTExit();
		exit(-1);
	}

/* initialize */
	RXInit();			/* Initialize RX */
#ifndef DEMO
	TXQInit();			/* Initialize TX */
	AX25_Init();		/* Initialize AX25 LAPB */
#endif /* demo */
	HeardInit();		/* Initialize the heard lists */
	InitCapture();		/* Initialize capture */

	if (!AutoMode)
		wait();


	time(&StartTime);
	normal();
	clrscr();
	StatusLine();

#ifdef TRACE
	LogInit();			/* Initialize the log */
	DebugMode = FALSE;	/* Initialize DebugMode to FALSE since	*/
	DoDebug();			/* DoDebug() will flip it to TRUE.		*/
#endif

#ifdef DEBUG
	PassMode = FALSE;
#endif

/* crank up timers */
#ifndef DEMO
	StartBeacon();
#endif
	SoundEnd = 0;

/* doit! */
	MainLoop();
	if (Capturing())			/* close capture file */
		DoCapture();

/* bye, bye, clean things up...  */
	CRTExit();
#ifdef TRACE
	DumpLog();			/* dump the debug log */
#endif
}


/* Attempt to open a file. If unable to do so, this routine returns NULL */

FILE *OpenFile(char *prompt, char *fname, int maxlen)
{
char *s, text[2];
FILE *fp;

	fp = NULL;
	s = fname;

	/* Get NUL terminated string of maxlen and place in fname */
	if (GetInput(prompt, s, maxlen))
	{
		/* Strip blanks and non-printable characters from start */
		while (*s && (*s == ' ' || !isprint(*s)) )
			++s;

		strcpy(fname, s);	/* Copy name with no junk at start */

		/* If we are not pointing at a NUL, strip junk off end */
		if (*fname)
		{
			s = &fname[strlen(fname) - 1];
			while (*s == ' ' || !isprint(*s) )
				--s;
			s[1] = '\0';
		}

		/* If only thing left is a NUL, we didn't get a valid filename */
		if (*fname == '\0')
			return(NULL);

		strupr(fname);
		if (access(fname, 0) == -1)		/* Does file already exist? */
			fp = fopen(fname, "w");
		else
		{
			if (!GetInput("File already exists.  Append, Overwrite, or Cancel? [a/o/C]", text, 2))
				return(NULL);
			*text = toupper(*text);
			if (*text != 'A' && *text != 'O')
				return(NULL);
			if (*text == 'A')
				fp = fopen(fname, "a");		/* Append to old file */
			else
			{
				if (!GetInput("Current file contents will be lost. Are you sure? [y/N]", text, 2))
					return(NULL);
				if (toupper(*text) != 'Y')
					return(NULL);
				fp = fopen(fname, "w");	/* Erase old file and start a new one */
			}
		}
	}

	if (fp == NULL)
	{
		gotoxy(1, 24);
		uprintf(InvAttr, "  Error opening '%s' : %s  ", fname, sys_errlist[errno]);
	}

	return(fp);
}


/*  time2str()
	Convert a time value into an ASCII string
*/
void time2str(time, str)
long time;
char *str;
{
	int	hours, minutes, secs;

	secs = (int)(time % 60L);
	time /= 60L;
	minutes = (int)(time % 60L);
	time /= 60L;
	hours = (int)(time % 24L);
	time /= 24L;
	if (time == 0L)
		sprintf(str, "%02d:%02d:%02d", hours, minutes, secs);
	else
		sprintf(str, "%d days %02d:%02d:%02d", (int)time, hours, minutes, secs);
}
