/*
	pmptest.c -- A program to test the hardware (modem) setup for PMP

  Poor Man's Packet (PMP)
  Copyright (c) 1991 by Andrew C. Payne    All Rights Reserved.

  Permission to use, copy, modify, and distribute this software and its
  documentation without fee for NON-COMMERCIAL AMATEUR RADIO USE ONLY is hereby
  granted, provided that the above copyright notice appear in all copies.
  The author makes no representations about the suitability of this software
  for any purpose.  It is provided "as is" without express or implied warranty.

	August, 1989
	Andrew C. Payne
*/

/* ----- Includes ----- */
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>
#include <bios.h>
#include <ctype.h>
#include <string.h>

#define EXTERN	extern
#include "types.h"
#include "ports.h"
#include "keys.h"

#define VIDEO	0x10
#define TRUE	1
#define FALSE	0

#define BITTIME 1988		/* 1200 baud bit time in ticks */
#define CAPSIZE	2400		/* number of capture transitions */

extern word timer(void);
extern word transit(word);
extern word waittrans(word);
extern void InitParameters(void);
extern int	ReadParameters(void);

extern int	monochrome;
extern int	NormalAttr;
extern int	InvAttr;
extern int	BrightAttr;
extern char ParamFname[];	/* name of parameter file */

int	TXState;				/* Current state of the TX data line */

int	prevRXCarrier;			/* prev values to minimize screen updates */
int	prevRXLevel;

int	capbuf[CAPSIZE];

/* ----- Low Level Screen Control ----- */

/* v_setctype(start, end)
	Set the cursor size:  start and end.
*/
void cdecl v_setctype(int s, int e)
{
	_AH = 1;
	_CH = s;
	_CL = e;
	geninterrupt(VIDEO);
}

/* curoff()
	Turns the cursor off.
*/
void cdecl curoff(void)
{
	v_setctype(0x0f, 0x0f);
}

/* curon()
	Turns the cursor on.
*/
void cdecl curon(void)
{
	if (monochrome)
		v_setctype(12, 13);
	else
		v_setctype(6, 7);
}


/* putstring(x, y, len, attr, string)
	Given an absolute screen position, a buffered length, and a string,
write string to screen. (FAST!!)
*/
void putstring(int x, int y, int len, char attr, char *s)
{
	char	buf[80*2];		/* buffer for string */
	char	*p;
	int	i;

/* fill buffer with screen data */
	p = buf;
	for (i = 0; i < len; ++i)
	{
		if (*s)
			*p++ = *s++;
		else
			*p++ = ' ';	/* buffer with spaces */

		*p++ = attr;		/* character attribute */
	}

/* write string to screen */
	puttext(x, y, x+len-1, y, buf);
}

void TitleScreen(void)
{
	clrscr();
	cprintf("Ŀ\r\n");
	cprintf(" PMP Test  Version 0.7SYB   Alignment and Test Program for Poor Man's Packet \r\n");
	cprintf("          Copyright (c) 1990 Andrew C. Payne   All rights reserved.          \r\n");
	cprintf("          Additional modifications courtesy of Kevin Cozens, VE3SYB          \r\n");
	cprintf("\r\n");

	cprintf("  Commands:\r\n");
	cprintf("    [ESC]     to exit\r\n");
	cprintf("    [SPACE]   to toggle transmit on and off\r\n");
	cprintf("    [F1]      for on-air modem alignment mode\r\n");
	cprintf("    [F2]      to toggle transmit data level\r\n");
	cprintf("    [F3]      to select 600hz transmit modulation\r\n");
#ifdef	ANDY
	cprintf("    [F4]      for loopback alignment\r\n");
#endif
	gotoxy(1, 22);
	cprintf("     Carrier         Receive           Transmit         Push To\r\n");
	cprintf("     Detect           Data               Data             Talk\r\n");
}

/* ----- Low Level I/O ----- */

/* RXCarrier()
	Returns TRUE if receive carrier detected.
*/
int RXCarrier(void)
{
	register byte	x;

	x = inportb(CDPort) & CDBit;
	return CDLevel ? x : !x;
}

/* RXLevel()
	Returns the current level of the RX data line.
*/
int RXLevel(void)
{
	return (inportb(RXPort) & RXBit) == RXBit;
}

/* TXKey(state)
	Sets the transmitter state to the state given (TRUE = 1 = keyup).
	Updates screen status.
*/
void TXKey(int state)
{
	PTTLevel = state;

	if (state)
	{
		outportb(PTTPort, inportb(PTTPort) | PTTBit);
		putstring(59, 24, 4, 0x70, " TX ");
	}
	else
	{
		outportb(PTTPort, inportb(PTTPort) & ~PTTBit);
		putstring(59, 24, 4, 0, "    ");
	}
}

/* SetTXLevel(x)
	Sets the TX data level to the level specified. Updates screen status.

	NOTE: In the following routine, we get the data byte at TXPort and
	clear the TXBit. The XOR of 'TXBit' and 'TXLevel' creates a bit state
	with the correct logical sense (in case a serial port is being used.
	The final XOR or OR then uses this bit state to properly set or clear
	the TX data bit. In this way, we support modems connected via parallel
	and serial ports. The TX data level needs to be inverted for the serial
	port.
*/
void SetTXLevel(int x)
{
	TXState = x;

	if (TXState)
	{
		outportb(TXPort, (inportb(TXPort) & ~TXBit) ^ TXBit ^ TXLevel);
		putstring(42, 24, 4, 0x70, " TD ");
	}
	else
	{
		outportb(TXPort, (inportb(TXPort) | TXBit) ^ TXBit ^ TXLevel);
		putstring(42, 24, 4, 0, "    ");
	}
}

/* UpdateScreen()
	Update the screen status to reflect the current Carrier Detect
	level and the current receive level.
*/
void UpdateScreen(void)
{
	int	t;

/* show status of CD */
	if ((t = RXCarrier()) != prevRXCarrier)
	{
		prevRXCarrier = t;
		if (prevRXCarrier)
			putstring(7, 24, 4, 0x70, " CD ");
		else
			putstring(7, 24, 4, 0, "    ");
	}

/* show receive level */
	if ((t = RXLevel()) != prevRXLevel)
	{
		prevRXLevel = t;
		if (prevRXLevel)
			putstring(23, 24, 4, 0x70, " RD ");
		else
			putstring(23, 24, 4, 0, "    ");
	}
}

/* SquareWave()
	Generate a 600hz square wave on the output until a key is pressed.
*/
void SquareWave(void)
{
	word	from;

/* show what is going on on the screen */
	gotoxy(1, 14);
	cprintf(" 600hz Transmit Ŀ\r\n");
	cprintf("                                                                             \r\n");
	cprintf("                      Transmitting 600hz square wave                         \r\n");
	cprintf("                                                                             \r\n");
	cprintf("                                                                             \r\n");
	cprintf(" Press Any Key To Exit \r\n");

	from = timer();
	while (!keypressed())		/* wait till key interrupt */
		transit(from -= BITTIME);

	SetTXLevel(TXState);

	(void)getkey();					/* gobble */
	putstring(1, 14, 79, 0, "");	/* clear the lines */
	putstring(1, 15, 79, 0, "");
	putstring(1, 16, 79, 0, "");
	putstring(1, 17, 79, 0, "");
	putstring(1, 18, 79, 0, "");
	putstring(1, 19, 79, 0, "");
}

/* Alignment()
	Waits for carrier detect (or user keystroke), and captures a
	few transitions of data.  Computes and displays alignment information
	from transition timing.
*/
void Alignment(void)
{
	int		i;
	word	t, t1, t2;
	int		actual, delta;
	long	deltahi, deltalo;
	int		deltahict, deltaloct;
	char	s[100];

/* show what is going on on the screen */
	gotoxy(1, 14);
	cprintf(" Modem Alignment Ŀ\r\n");
	cprintf("   Adjust modem for lowest numbers possible  (see manual for more info)      \r\n");
	cprintf("                                                                             \r\n");
	cprintf("                                                                             \r\n");
	cprintf("                                                                             \r\n");
	cprintf(" Press Any Key To Exit \r\n");

	for ( ; ; )
	{
		while (!RXCarrier() && !keypressed())
			;

		if (keypressed())
		{
			(void)getkey();					/* gobble key */
			putstring(1, 14, 79, 0, "");	/* clear the lines */
			putstring(1, 15, 79, 0, "");
			putstring(1, 16, 79, 0, "");
			putstring(1, 17, 79, 0, "");
			putstring(1, 18, 79, 0, "");
			putstring(1, 19, 79, 0, "");
			return;
		}
		UpdateScreen();

/* start capture */
		deltahi = deltalo = deltahict = deltaloct = 0;
		t1 = timer();
		disable();

		for (i = 0; i < CAPSIZE; ++i)
		{
			t2 = waittrans(t1-20000);
			t = t1 - t2;
			t1 = t2;
			actual = ((t + 994) / BITTIME) * BITTIME;
			delta = actual - t;
			if (RXLevel())
			{
				deltahi += delta;
				deltahict++;
			}
			else
			{
				deltalo += delta;
				deltaloct++;
			}
			if (!RXCarrier())
				break;
		}
		enable();
		UpdateScreen();
		sprintf(s, "Low -> High error: %-5ld%%    High -> Low error: %-5ld%%", 
			(deltahi / deltahict) * (long)100 / BITTIME, 
			(deltalo / deltaloct) * (long)100 / BITTIME);
		putstring(16, 17, 60, 7, s);
		delay(1000);
	}
}

#ifdef	ANDY
/* Loopback()
	Does a loopback alignment.  Assumes that the transmit audio is
	looped back into the receive audio.
*/
void Loopback()
{
	word	t, t1, d;
	int	i, j, k;
	char	s[100];
#ifdef	ANDY
	long	deltalo, deltahi;
#endif

/* show what's going on */
	gotoxy(1, 14);
	cprintf(" Loopback Alignment Ŀ\r\n");
	cprintf("   Adjust modem for lowest numbers possible  (see manual for more info)      \r\n");
	cprintf("                                                                             \r\n");
	cprintf("                                                                             \r\n");
	cprintf("                                                                             \r\n");
	cprintf(" Press Any Key To Exit \r\n");

/* loop until keypressed */
	while (!keypressed())
	{
		SetTXLevel(0);
		delay(100);
		disable();
		t = timer();
		j = RXLevel();
		transit(t -= BITTIME);
		waituntil(t -= 50000);
		waituntil(t -= 50000);
		i = RXLevel();
		enable();
		delay(100);
		sprintf(s, "%d %d", j, i);
		putstring(15, 17, 60, 7, s);
	}

#ifdef ANDY
	sprintf(s, "   ", j, i);
	putstring(15, 17, 60, 7, s);
	(void)getkey();

/* do a second-long sample */
	for (i = 0; i < 600; ++i)
	{
/* do a low->high transition */
		transit(t -= BITTIME);		/* do a transition */
		deltahi = t - waittrans(t - 50000);
		d = RXLevel();
		if (d)
			deltahi++;

/* do a high-low transition */
		transit(t -= BITTIME);		/* do a transition */
		deltalo = t - waittrans(t - 50000);
		d = RXLevel();
		if (!d)
			deltalo++;
	}
	enable();

	sprintf(s, "Low -> High error:  %-5ld%%    High -> Low error:  %-5ld%%", deltahi, deltalo);
	putstring(15, 17, 60, 7, s);
#endif

	SetTXLevel(TXState);

	(void)getkey();					/* gobble the key */
	putstring(1, 14, 79, 0, "");		/* clear the lines */
	putstring(1, 15, 79, 0, "");
	putstring(1, 16, 79, 0, "");
	putstring(1, 17, 79, 0, "");
	putstring(1, 18, 79, 0, "");
	putstring(1, 19, 79, 0, "");
}
#endif

/* HandleKey()
	Handles user's keystrokes.
*/
int HandleKey(void)
{
	switch (getkey())
	{
		case ESC:
		case ALTX:
			return TRUE;		/* done! */
		case SPC:
			TXKey(!PTTLevel);
			break;
		case F1:			/* on-air alignment */
			Alignment();
			break;
		case F2:			/* toggle TX level */
			SetTXLevel(!TXState);
			break;
		case F3:			/* 600hz square wave */
			SquareWave();
			break;
		case F4:			/* loopback test */
#ifdef	ANDY
			Loopback();
#endif
			break;
	}
	return FALSE;
}

/* usage()
	Show the command line usage.
*/
void usage(void)
{
	printf("Usage:  PMPTEST [-p <paramfile>]\n");
}

/* ----- Main Program ----- */

void main(int argc, char **argv)
{
	int	i;

/* process command line parameters */
	strcpy(ParamFname, "PMP.CFG");
	for (i = 1; i < argc; i++)
	{
		if (argv[i][0] != '-')
		{
			usage();
			exit(-1);
		}
		if (tolower(argv[i][1]) != 'p')
		{
			printf("Unknown switch: %s\n", argv[i]);
			usage();
			exit(0);
		}
		if (argv[i][2] != '\0')
			strcpy(ParamFname, argv[i]+2);
		else
		{
			if (i < argc - 1)
				strcpy(ParamFname, argv[++i]);
			else
			{
				usage();
				exit(0);
			}
		}
	}

	clrscr();
	curoff();
	NormalAttr = LIGHTGRAY;
	InvAttr = LIGHTGRAY << 4;
	BrightAttr = NormalAttr | 0x08;
	TitleScreen();

/* Initialize */
	InitParameters();
	if (ReadParameters())
		exit(-1);			/* Error reading parameter file */

	monochrome = TRUE;
	TXKey(FALSE);
	SetTXLevel(0);

/* loop, handling keystrokes and showing current status */
	while (TRUE)
	{
		if (keypressed())
		{
			if (HandleKey())
				break;
		}
		UpdateScreen();
	}

/* clean up and exit */
	clrscr();
	curon();
	TXKey(FALSE);		/* drop transmit */
}
