/*
	yappxfer.c -- YAPP(tm) binary transfer protocol
  
  Poor Man's Packet (PMP)
  
  Permission to use, copy, modify, and distribute this software and its
  documentation without fee for NON-COMMERCIAL AMATEUR RADIO USE ONLY is
  hereby granted, provided that the above copyright notice appear in all
  copies.  The author makes no representations about the suitability of
  this software for any purpose.  It is provided "as is" without express
  or implied warranty.

  This implements the YAPP(tm) binary transfer protocol (or at least
  a subset of the full protocol - this version does not include the
  server commands for automated transfer.)
  
  This is a modified version of the actual code used in YAPP for the
  IBM PC and compatibles.  Some lines have been deleted that handled
  functions such as displaying the status of the transfer, and checking
  for an abort from the keyboard.

	August, 1992	Kevin Cozens
	This is an translation and adaptation of the YAPP transfer routines
	written and copyright in 1986 by Jeffry B. Jacobsen.
*/

#include	<stdio.h>
#include	<malloc.h>
#include	<string.h>
#include	"pmp.h"

#define 	A_SOH		('\001')
#define 	A_STX		('\002')
#define 	A_ETX		('\003')
#define 	A_EOT		('\004')
#define 	A_ENQ		('\005')
#define 	A_ACK		('\006')
#define 	A_DLE		('\020')
#define 	A_NAK		('\025')
#define 	A_CAN		('\030')


typedef enum
{
	S1, S2, SH, SD, SE, ST, R, RH, RD, Abort, CW, C, Start
} States;

typedef enum
{
	UK, R1, RF, SI, HD, DT, EF, ET, NR, CN, CA, RI, TX, UU, TM, AF, AT
} Packet_Types;


static char *stdesc[] = {
	"SendInit",		"SendInit1",	"SendHeader",	"SendData",
	"SendEof",		"SendEOT",		"RcvWait",		"RcvHeader",
	"RcvData",		"SndABORT",		"WaitAbtAck",	"RcdABORT",
	"Start"
};

static int sendinit_count;
static char xferhdr[80];
static long xfercnt;
static int xferok;
static enum States state;
static enum Packet_Types ptype;

static char pkbuff[256];
static int pklen;
static FILE *pkfile;
static char pkfname[30];

static int timeout;		/* ~~~~~ */

/* ------------------------------------------------------------------- */

static void set_timer(int waittime)
{
}


/* check for character ready at TNC */
static int inready(void)
{
	return(FALSE);
}


static void disppacket(void)
{
}


static void showmsg(int msgnum)
{
}


static void showheader(void)
{
}


static char recvchar(void)
{
	return('\0');
}


static void showbytes(void)
{
}


static void escmsg(int msgnum)
{
}


/* display state */
static void showstate(void)
{
}


/* send string to TNC */
static void xmitstr(char c1, char c2, char *str, int str_len)
{
char *p;

	p = malloc(str_len + 2);
	p[0] = c1;
	p[1] = c2;
	strncpy(&p[2], str, str_len);
	LinkSend(p, str_len + 2);
	free(p);
}


/* ------------------------------------------------------------------- */


/* wait 20 seconds or 120 seconds for a character */

static int waitready(void)
{
	int ret;

	ret = FALSE;
	if (state == S1 || state == S2)
		set_timer(20);		/* 20 seconds to timeout */
	else
		set_timer(120);		/* 120 seconds to timeout */
	do {
		if (timeout)
		{
			ptype = TM;		/* timeout checks timer value which is */
			ret = TRUE;		/* decremented towards 0 every second */
			return (ret);
		}
	} while (!inready());	/* inready checks for character ready at TNC */
	return (ret);
}


static void getpkstr(void)
{
	int i;

	if (waitready() == TRUE)
		return;

	pklen = (int)recvchar();
	if (ptype == DT && pklen == 0)
		pklen = 256;
	if (pklen == 0)
		return;

	for (i = 0; i < pklen; ++i)
	{
		if (waitready())
			return;
		pkbuff[i] = recvchar();
	}
}


static void getpack(void)
{
	ptype = UK;
	if (waitready())
		return;

	switch (recvchar())
	{
	case A_ACK:
		if (waitready())
			return;
		switch (recvchar())
		{
		case 1:
			ptype = R1;
			break;
		case 2:
			ptype = RF;
			break;
		case 3:
			ptype = AF;
			break;
		case 4:
			ptype = AT;
			break;
		case 5:
			ptype = CA;
			break;
		default:
			break;
		}
		break;
	case A_ENQ:
		if (waitready())
			return;
		switch (recvchar())
		{
		case 1:
			ptype = SI;
			break;
		case 2:
			ptype = RI;
			break;
		default:	/* unimplemented command */
			ptype = UU;
			break;
		}
		break;
	case A_SOH:
		ptype = HD;
		getpkstr();
		break;
	case A_STX:
		ptype = DT;
		getpkstr();
		break;
	case A_ETX:
		if (waitready())
			return;
		if (recvchar() == 1)
			ptype = EF;
		break;
	case A_EOT:
		if (waitready())
			return;
		if (recvchar() == 1)
			ptype = ET;
		break;
	case A_NAK:
		ptype = NR;
		getpkstr();
		break;
	case A_CAN:
		ptype = CN;
		getpkstr();
		break;
	case A_DLE:
		ptype = TX;
		getpkstr();
		break;
	default:
		break;
	}
}


static void sendinit(void)
{
	if (state == S1)	/* send initialization */
	{
		sendinit_count = 0;
		xmitstr(A_ENQ, 1, NULL, 0);
	}
	else			/* send initialization retry */
	{
		if (++sendinit_count > 6)
		{
			state = Abort;
			showmsg(12);
			return;
		}
	}

	getpack();
	switch (ptype)
	{
	case TM:
		state = S2;
		break;
	case RI:
		state = S1;
		break;
	case R1:
		state = SH;
		break;
	case RF:
		state = SD;
		break;
	case CN:
		state = C;
		break;
	case NR:
		state = Start;
		break;
	case TX:
		disppacket();
		break;
	default:
		state = Abort;
		showmsg(13);	/* error message display */
		break;
	}
}


static void sendhdr(void)
{
	unsigned char stlen;
	char temp[80];
	long filesize;

	fseek(pkfile, 0L, 2);
	filesize = ftell(pkfile);
	sprintf(xferhdr, "%s%ld", pkfname, filesize);
	showheader();			/* display */
	stlen = strlen(temp);
	xmitstr(A_SOH, stlen, temp, stlen);

	getpack();
	switch (ptype)
	{
	case RF:
		state = SD;
		break;
	case CN:
		state = C;
		break;
	case NR:
		state = Start;
		break;
	case TX:
		disppacket();
		break;
	default:
		state = Abort;
		if (ptype == TM)
			showmsg(12);
		else
			showmsg(13);
		break;
	}
}


static void senddata(void)
{
	int cnt;
	unsigned char bte;
	char temp[256];

	if (inready())			/* we shouldn't be getting a packet   */
	{						/* unless they sent a Cancel or Text */
		getpack();
		if (ptype == CN)
		{
			state = C;
			return;
		}
		else
		{
			if (ptype == TX)
				disppacket();
			else
			{
				printf("Unexpected packet type during Send!\n");
				state = Abort;
				return;
			}
		}
	}

	cnt = fread(temp, 1, 256, pkfile);
	if (cnt != 0)
	{
		if (cnt == 256)
			bte = 0;
		else
			bte = cnt;
		xmitstr(A_STX, bte, temp, cnt);
	}
	xfercnt += (long)cnt;
	if (cnt < 256)
		state = SE;
}


static void sendeof(void)
{
	xmitstr(A_ETX, 1, NULL, 0);

	getpack();
	switch (ptype)
	{
	case AF:
		state = ST;
		break;
	case TX:
		disppacket();
		break;
	default:
		state = Abort;
		if (ptype == TM)
			showmsg(12);
		else
			showmsg(13);
		break;
	}
}


static void sendeot(void)
{
	xmitstr(A_EOT, 1, NULL, 0);

	getpack();
	switch (ptype)
	{
	case AT:			/* Ack ok */
		state = Start;
		break;
	case TX:
		disppacket();
		break;
	default:
		state = Start;	/* They sent AF - so don't worry about it */
		break;
	}
}


static void receive(void)
{
	getpack();
	switch (ptype)
	{
	case SI:
		showmsg(1);
		xmitstr(A_ACK, 1, NULL, 0);
		state = RH;
		break;
	case CN:
		state = C;
		break;
	case TX:
		disppacket();
		break;
	default:
		state = Abort;
		if (ptype == TM)
			showmsg(12);
		else
			showmsg(13);
		break;
	}
}


static void rcvhdr(void)
{
	getpack();
	switch (ptype)
	{
	case HD:
		strncpy(xferhdr, pkbuff, pklen);
		showheader();
		xmitstr(A_ACK, 2, NULL, 0);
		state = RD;
		break;
	case SI:
		state = RH;
		break;
	case CN:
		state = C;
		break;
	case ET:
		xmitstr(A_ACK, 4, NULL, 0);
		state = Start;
		break;
	case TX:
		disppacket();
		break;
	default:
		state = Abort;
		if (ptype == TM)
			showmsg(12);
		else
			showmsg(13);
		break;
	}
}


static void rcvdata(void)
{
	getpack();
	switch (ptype)
	{
	case DT:
		fwrite(pkbuff, pklen, 1, pkfile);
		xfercnt += (long)pklen;
		showbytes();
		state = RD;
		break;
	case EF:
		fclose(pkfile);
		xferok = TRUE;
		showmsg(8);
		xmitstr(A_ACK, 3, NULL, 0);
		state = RH;
		break;
	case CN:
		state = C;
		break;
	case TX:
		disppacket();
		break;
	default:
		state = Abort;
		if (ptype == TM)
			showmsg(12);
		else
			showmsg(13);
		break;
	}
}


static void cancel(void)
{
	xmitstr(A_CAN, 0, NULL, 0);
	state = CW;
}


static void canwait(void)
{
	escmsg(10);

	getpack();
	switch (ptype)
	{
	case CA:
		state = Start;
		break;
	case CN:
		xmitstr(A_ACK, 5, NULL, 0);
		break;
	case TM:
		state = Start;
		break;
	case UK:
		state = Start;
		break;
	case TX:
		disppacket();
		break;
	default:
		break;
	}
}


static void canrecd(void)
{
	showmsg(11);
	xmitstr(A_ACK, 5, NULL, 0);
	state = Start;
}


static void xfer(void)
{
	AX25_Control.type = BINARY;

	xfercnt = 0L;

	do {
		showstate();	/* display state */
		switch (state)
		{
		case S1:			/* send initialization */
		case S2:		/* send initialization retry */
			sendinit();
			break;
		case SH:
			sendhdr();
			break;
		case SD:
			senddata();
			break;
		case SE:
			sendeof();
			break;
		case ST:
			sendeot();
			break;
		case R:
			receive();
			break;
		case RH:
			rcvhdr();
			break;
		case RD:
			rcvdata();
			break;
		case Abort:
			cancel();
			break;
		case CW:
			canwait();
			break;
		case C:
			canrecd();
			break;
		default:
			break;
		}
	} while (state != Start);

	printf("\007");		/* bell */
}


static void upload(void)
{
	if (!GetInput("Enter name of file to upload: ", pkfname, sizeof(pkfname)))
		return;

	pkfile = fopen(pkfname, "rb");
	fseek(pkfile, 0L, 0);

	state = S1;
	xfer();
	fclose(pkfile);
}


static void download(void)
{
	if (!GetInput("Enter name of file to download: ", pkfname, sizeof(pkfname)))
		return;

	pkfile = fopen(pkfname, "wb");
	fseek(pkfile, 0L, 0);

	state = R;
	xferok = FALSE;
	xfer();

	if (!xferok)
	{
		fclose(pkfile);
		remove(pkfname);
	}
}
